/* ==================================================================
   File:        config.cpp
   Author:      Steve Monks
   Description:
	Configuration module

 	$Id: config.cpp,v 1.6 2006/04/09 19:51:52 Steve Exp $
	$Log: config.cpp,v $
	Revision 1.6  2006/04/09 19:51:52  Steve
	Version 3.6
	Added logarithmic volume control
	Added diagnostic logging
	
	Revision 1.5  2006/02/04 20:59:44  Steve
	Initial import of readme.html
	
	Added gapless mode (for evaluation purposes)
	
	Revision 1.4  2006/01/30 21:07:28  Steve
	Added protection against daft values to config stuff
	Fixed config dialog so that KSCore is only recreated when settings actually change
	Changed buffer size readouts to KB in config dialog
	Fixed potential problem with lost volume settings when KScore is deleted and recreated
	
	Revision 1.3  2006/01/29 21:33:56  Steve
	Added an adjustment for the number of output buffers
	Removed buffer size radio buttons and replaced with sliders
	Now deletes and recreates the KSCore object when a new track starts after changing the settings
	
	Revision 1.2  2006/01/03 21:46:13  Steve
	Initial import of V3.0.
	
	This is a complete rewrite of the plugin and Kernel streaming code.
	The only code retained from the previous 2.x version is the Microsoft
	sample code and the filter enumeration.
	
	Revision 1.1.1.1  2006/01/02 12:16:11  Steve
	Initial import
	
	Revision 1.3  2005/12/27 01:20:01  Steve
	Added Oversampling for <48KHz 16 bit audio
	Significant re-org and clean up of code
	
================================================================== */
#include <windows.h>
#include "stdio.h"
#include "config.h"

static char app_name[] = "Kernel Streaming";


void Config::ReadString(char *name,char *data, int mlen)
{
	char buf[2048];
	strcpy(buf,data);
	GetPrivateProfileString(app_name,name,buf,data,mlen,file_name);
}


void Config::ReadInt(char *name,int *v)
{
	int def_v;

	def_v = *v;
	*v = GetPrivateProfileInt(app_name,name,def_v,file_name);
}



void Config::WriteString(char *name, char *data)
{
	WritePrivateProfileString(app_name,name,data,file_name);
}


void Config::WriteInt(char *name, int v)
{
	char txt[16];
	sprintf(txt,"%d",v);
	WritePrivateProfileString(app_name,name,txt,file_name);
}




Config::Config(void)
{
	char *dot;

	GetModuleFileName(NULL,file_name,sizeof(file_name));
	dot = strchr(file_name,'.');
	if (dot) {
		strcpy(dot,".ini");
	}

	Defaults();

	// get current settings from file (if any)
	Read();
}


void Config::Defaults(void)
{
	out_name[0]        = 0;
	volume_enabled     = FALSE;
	gapless_enabled    = FALSE;
	input_buffer_size  = 65536;
	output_buffer_size = 8192;
	output_buffer_cnt  = 8;
	logging_enabled    = FALSE;
	logarithmic_volume = TRUE;
}

void Config::Read(void)
{

	ReadString("output_device",	out_name,sizeof(out_name));
	ReadInt("volume_enabled",		&volume_enabled		);
	ReadInt("gapless_enabled",		&gapless_enabled		);
	ReadInt("input_buffer_size",	&input_buffer_size	);
	ReadInt("output_buffer_size",	&output_buffer_size	);
	ReadInt("logging_enabled",		&logging_enabled		);
	ReadInt("logarithmic_volume",	&logarithmic_volume	);

	input_buffer_size  = 1024 * (max(4,input_buffer_size  / 1024));	// restrict to 1K boundaries and a minimum of 4K
	output_buffer_size = 1024 * (max(2,output_buffer_size / 1024));	// restrict to 1K boundaries and a minimum of 2K
	output_buffer_cnt  = max(2,min(8,output_buffer_cnt));					// restrict to between 2 and 8
}

void Config::Write(void)
{
	WriteString("output_device",	out_name				);
	WriteInt("volume_enabled",		volume_enabled		);
	WriteInt("gapless_enabled",	gapless_enabled	);
	WriteInt("input_buffer_size",	input_buffer_size	);
	WriteInt("output_buffer_size",output_buffer_size);
	WriteInt("output_buffer_cnt",	output_buffer_cnt	);
	WriteInt("logging_enabled",	logging_enabled	);
	WriteInt("logarithmic_volume",logarithmic_volume);
}
